require 'spec_helper'

describe ComplianceController do
  def fixture(name)
    File.read(File.join(File.dirname(__FILE__), "../fixtures/reports/#{name}.yaml"))
  end

  describe "#build_node_summary" do
    before :each do
      Summarizer.stubs(:delay).returns(Summarizer)

      @foo_report = Report.create_from_yaml(fixture('foo'))
      @bar_report = Report.create_from_yaml(fixture('bar'))
      @baz_report = Report.create_from_yaml(fixture('baz'))

      foo_baseline_report = Factory(:inspect_report, :host => 'foo')
      bar_baseline_report = Factory(:inspect_report, :host => 'bar')
      baz_baseline_report = Factory(:inspect_report, :host => 'baz')

      @foo_baseline = NodeBaseline.create_from_report!(foo_baseline_report, @foo_report.time)
      @bar_baseline = NodeBaseline.create_from_report!(bar_baseline_report, @bar_report.time)
      @baz_baseline = NodeBaseline.create_from_report!(baz_baseline_report, @baz_report.time)

      Time.zone.stubs(:now).returns(@foo_report.time)

      Summarizer.queue_diff_summarization
    end

    it "should collect resource and node counts for groups" do
      group = NodeGroup.generate!
      group.nodes = [@bar_report.node, @baz_report.node]
      group.save!

      get :index
      assigns(:node_summary).groups.should == [OpenStruct.new(
        :group_id                  => group.id,
        :name                      => group.name,
        :node_count                => 2,
        :failed_resource_count     => 0,
        :unreviewed_resource_count => 6,
        :accepted_resource_count   => 0,
        :rejected_resource_count   => 0,
        :completed_node_count      => 2,
        :completeness              => 100
      )]

      @bar_baseline.differences.each {|diff| diff.accept_change }
      @baz_baseline.differences.each {|diff| diff.reject_change }

      get :index
      assigns(:node_summary).groups.should == [OpenStruct.new(
        :group_id                  => group.id,
        :name                      => group.name,
        :node_count                => 2,
        :failed_resource_count     => 0,
        :unreviewed_resource_count => 0,
        :accepted_resource_count   => 3,
        :rejected_resource_count   => 3,
        :completed_node_count      => 2,
        :completeness              => 100
      )]
    end

    it "should calculate the diff completion per group" do
      finished_group = NodeGroup.generate!(:name => 'finished')
      finished_group.nodes = [@foo_report.node, @bar_report.node]
      finished_group.save!

      unfinished_group = NodeGroup.generate!(:name => 'unfinished')
      unfinished_group.nodes = [@baz_report.node]
      @baz_baseline.reload
      @baz_baseline.update_attributes!(:diffed => false)
      unfinished_group.save!

      get :index

      expected = [['finished', 2, 100], ['unfinished', 0, 0]]
      assigns(:node_summary).groups.map{|g| [g.name, g.completed_node_count, g.completeness]}.should =~ expected
    end

    it "should collect resource counts for nodes not in a group" do
      group = NodeGroup.generate!
      group.nodes = [@baz_report.node]
      group.save!

      get :index
      assigns(:node_summary).nodes.should == [
        OpenStruct.new(
          :node_id                   => @bar_report.node.id,
          :name                      => @bar_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 3,
          :accepted_resource_count   => 0,
          :rejected_resource_count   => 0
        ),
        OpenStruct.new(
          :node_id                   => @foo_report.node.id,
          :name                      => @foo_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 3,
          :accepted_resource_count   => 0,
          :rejected_resource_count   => 0
        )
      ]

      @bar_baseline.differences.each {|diff| diff.accept_change }
      @foo_baseline.differences.each {|diff| diff.reject_change }

      get :index
      assigns(:node_summary).nodes.should == [
        OpenStruct.new(
          :node_id                   => @bar_report.node.id,
          :name                      => @bar_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 0,
          :accepted_resource_count   => 3,
          :rejected_resource_count   => 0
        ),
        OpenStruct.new(
          :node_id                   => @foo_report.node.id,
          :name                      => @foo_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 0,
          :accepted_resource_count   => 0,
          :rejected_resource_count   => 3
        )
      ]
    end

    it "should excludes groups with no data from totals" do
      group_with_data = NodeGroup.generate!(:nodes => [@foo_report.node, @bar_report.node, @baz_report.node])

      group = NodeGroup.generate!
      3.times { group.nodes << Node.generate! }
      group.save!

      get :index

      response.should be_success

      assigns[:node_summary].node_count.should == 3
    end

    it "should exclude nodes with no data" do
      3.times { Node.generate! }

      get :index

      response.should be_success

      assigns[:node_summary].nodes.should == [
        OpenStruct.new(
          :node_id                   => @bar_report.node.id,
          :name                      => @bar_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 3,
          :accepted_resource_count   => 0,
          :rejected_resource_count   => 0
        ),
        OpenStruct.new(
          :node_id                   => @baz_report.node.id,
          :name                      => @baz_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 3,
          :accepted_resource_count   => 0,
          :rejected_resource_count   => 0
        ),
        OpenStruct.new(
          :node_id                   => @foo_report.node.id,
          :name                      => @foo_report.node.name,
          :failed_resource_count     => 0,
          :unreviewed_resource_count => 3,
          :accepted_resource_count   => 0,
          :rejected_resource_count   => 0
        )
      ]
    end

    it "should not double count nodes in multiple groups when computing the total group completeness" do
      group1 = NodeGroup.generate!
      group1.nodes = [@foo_report.node, @bar_report.node, @baz_report.node]
      group1.save!

      group2 = NodeGroup.generate!
      group2.nodes = [@foo_report.node]
      group2.save!

      @foo_baseline.reload
      @foo_baseline.update_attributes!(:diffed => false)

      get :index

      # 2 nodes of 3 are diffed. If we double counted foo, we would get 2 of 4.
      assigns[:node_summary].group_completeness.should == (100.0 * 2/3).floor
    end
  end
end
